#include "tisound.h"
#include "calcmod.h"
//#include "songplay.h"

static int hexcmp(char *ptr1, char *ptr2, int n) {
	unsigned char tmp1,tmp2;
	for (int i = 0; i < n; i++) {
		tmp1 = *ptr1 & 0xFF;
		tmp2 = *ptr2 & 0xFF;
		if (tmp1 != tmp2) {
			return 1;
		}
		ptr1++;
		ptr2++;
	}
	return 0;
}

static int null_run(unsigned long samples) {
	init();
	unsigned long moved = 0;
	audio.ptr1 = vars.ptr;
	audio.ptr2 = vars.ptr;
	audio = formats[vars.format].set_pointers(vars.ptr,audio);
	while ((!vars.eof) && (samples > 0)) {
		if ((audio.period[0] == 0) || (audio.period[1] == 0))
			audio = formats[vars.format].read_data(audio);
		vars.eof = audio.eof;
		if (audio.tf[0] == 0) {
			audio.stat[0] = -(audio.stat[0]);
			audio.tf[0] = audio.freq[0];
		}
		if (audio.tf[1] == 0) {
			audio.stat[1] = -(audio.stat[1]);
			audio.tf[1] = audio.freq[1];
		}
		audio.tf[0] -= 1;
		audio.tf[1] -= 1;
		audio.period[0] -= 1;
		audio.period[1] -= 1;
		samples--;
		moved++;
	}
	vars.going = TRUE;
	return (int) (moved / 44.1);
}

static int get_length(char *filename,int type,bool leaveopen = FALSE) {
	adata temp = {
		FALSE,
		-1,
		NULL,
		NULL,
		{0,0},
		{0,0},
		{0,0},
		{1,1}
	};
	unsigned long samples = 0;
       	unsigned int size = 0;
	char buf[1];
	unsigned char *ptr;
	hdata tmp1;
	tmp1 = formats[vars.format].head(filename);
	long offset = tmp1.offset + tmp1.datalen;
	FILE *file;
	file = fopen(filename,"rb");
	fseek(file,offset,SEEK_SET);
	while ((fread(buf,1,1,file)) == 1) {
		size++;
	}
	fseek(file,offset,SEEK_SET);
	ptr = (unsigned char *) malloc(size);
	if (leaveopen)
		vars.ptr = ptr;
	if ((fread((ptr),1,size,file)) != size) {
		fclose(file);
		return -1;
	}
	fclose(file);
	temp.ptr1 = ptr;
	temp.ptr2 = ptr;
	temp = formats[type].set_pointers(ptr,temp);
	while (!temp.eof) {
		if ((temp.period[0] == 0) || (temp.period[1] == 0))
			temp = formats[type].read_data(temp);
		temp.period[0]--;
		temp.period[1]--;
		samples++;
	}
	if (!leaveopen)
		free(ptr);
	return (int) (samples/44.1);
}

static void init(void) {
	vars.going = FALSE;
	vars.eof = FALSE;
	vars.seek = -1;
	audio.eof = FALSE;
	audio.ptr1 = NULL;
	audio.ptr2 = NULL;
	audio.freq[0] = 0;
	audio.freq[1] = 0;
	audio.tf = audio.freq;
	audio.period[0] = 0;
	audio.period[1] = 0;
	audio.stat[0] = 1;
	audio.stat[1] = 1;
	return;
}

static int filetype(char *filename) {
	char *readbuf;
	FILE *file;
	hdata tmp;
	for (int i = 0; i < NUM_FORMATS; i++) {
		if ((formats[i].checkfile(filename))) {
			tmp = formats[i].head(filename);
			if ((file = fopen(filename,"rb")) != NULL) {
				if (fseek(file,tmp.offset,SEEK_SET) == 0) {
					readbuf = (char *) malloc(tmp.datalen);
					fread(readbuf,1,tmp.datalen,file);
					if ((strncmp(tmp.data,readbuf,tmp.datalen)) == 0) {
						fclose(file);
						return i;
					}
					free(readbuf);
				}
				fclose(file);
			}
		}
	}
	return -1;
}

static int is_our_file(char *filename) {
	char *readbuf;
	FILE *file;
	for (int i = 0; i < NUM_FORMATS; i++) {
		if ((formats[i].checkfile(filename))) {
			hdata tmp = formats[i].head(filename);
			if ((file = fopen(filename,"rb")) != NULL) {
				if (fseek(file,tmp.offset,SEEK_SET) == 0) {
					readbuf = (char *) malloc(tmp.datalen);
					fread(readbuf,1,tmp.datalen,file);
					if ((strncmp(tmp.data,readbuf,tmp.datalen)) == 0) {
						fclose(file);
						vars.format = i;
						return TRUE;
					}
					free(readbuf);
				}
				fclose(file);
			}
		}
	}
	return FALSE;
}

static void seek(int time) {
	vars.seek = time;
	vars.eof = FALSE;
	while (vars.seek != -1)
		xmms_usleep(10000);
	return;
}

static void* play_loop(void *arg) {
	signed char data[BUF_BYTES];
	int i;
	while (vars.going) {
		for (i = 0; i < BUF_SAMPLES; i++) {
			if ((audio.period[0] == 0) || (audio.period[1] == 0))
				audio = formats[vars.format].read_data(audio);
			vars.eof = audio.eof;
			if (audio.tf[0] == 0) {
				audio.stat[0] = -(audio.stat[0]);
				audio.tf[0] = audio.freq[0];
			}
			if (audio.tf[1] == 0) {
				audio.stat[1] = -(audio.stat[1]);
				audio.tf[1] = audio.freq[1];
			}
			data[i*2] = (VOL * audio.stat[0]);
			data[(i*2)+1] = (VOL * audio.stat[1]);
			audio.tf[0]--;
			audio.tf[1]--;
			audio.period[0]--;
			audio.period[1]--;
		}
		tisound.add_vis_pcm(tisound.output->written_time(),FMT_S8,1,BUF_BYTES,data);
		if ((vars.eof) || (audio.eof)) {
			vars.going = FALSE;
			tisound.output->buffer_free();
			tisound.output->buffer_free();
			while (tisound.output->buffer_playing())
				xmms_usleep(10000);
			tisound.output->close_audio();
			free(vars.ptr);
//			pthread_join(play_thread,NULL);
			pthread_exit(NULL);
			return NULL;
		}
		while (tisound.output->buffer_free() < BUF_BYTES && vars.going)
			xmms_usleep(10000);
		if (vars.going)
			tisound.output->write_audio(data,BUF_BYTES);
		if (vars.seek != -1) {
			tisound.output->flush(null_run((unsigned long) (vars.seek * 44100)));
			vars.seek = -1;
		}
			
	}
	free(vars.ptr);
	pthread_exit(NULL);
	return NULL;
}

static void play (char *filename) {
	finfo tmp;
	vars.format = filetype(filename);
	tmp = formats[vars.format].fileinfo(filename);
	tisound.output->open_audio(FMT_S8, 44100, 2);
	tisound.set_info((char *) tmp.title,get_length(filename,vars.format,TRUE),(8 * 2 * 44100),44100,2);
	free(tmp.title);
	init();
	audio.ptr1 = vars.ptr;
	audio.ptr2 = vars.ptr;
	audio = formats[vars.format].set_pointers(vars.ptr,audio);
	vars.going = TRUE;
	pthread_create(&play_thread,NULL,play_loop,NULL);
	return;
}

static void stop(void) {
	if (vars.going) {
		vars.going = FALSE;
		tisound.output->close_audio();
		pthread_join(play_thread,NULL);
	}
	return;
}

static void pause(short status) {
	tisound.output->pause(status);
	return;
}

static int get_time(void) {
	if (!vars.going && !tisound.output->buffer_playing())
		return -1;
	return tisound.output->output_time();
}

static void song_info(char *filename, char **title, int *length) {
	int type = filetype(filename);
	finfo tmp = formats[type].fileinfo(filename);
	(*title) = tmp.title;
	vars.title = tmp.title;
	if (tmp.length == -1)
		tmp.length = get_length(filename,type,FALSE);
	(*length) = tmp.length;
	return;
}

extern "C" {
	InputPlugin *get_iplugin_info(void) {
		return &tisound;
	}
}






